"""Example 16 + Restrain."""

from Definitions import *

# . Header.
logFile.Header ( )

# . Define constants.
_ForceConstant = 2.0
angle          = 60.0

# . Define dihedral to restrain.
PhiIndices     = [ 4, 6,  8, 14 ]

# . Define the energy models.
mmModel = MMModelOPLS.WithParameterSet ( "bookSmallExamples" )
nbModel = NBModelFull.WithDefaults ( )

# . Generate the molecule.
molecule = ImportSystem ( os.path.join ( molPath, "bala_c7eq.mol" ) )
molecule.DefineMMModel ( mmModel )
molecule.DefineNBModel ( nbModel )
molecule.Summary ( )
molecule.Energy  ( )

# . Define restraints.
restraints = RestraintModel ( )
molecule.DefineRestraintModel ( restraints )

# . Redefine the restraint.
rModel    = RestraintEnergyModel.Harmonic ( angle, _ForceConstant, period = 360.0 )
restraint = RestraintDihedral.WithOptions ( energyModel = rModel     ,
                                                    point1      = PhiIndices[0] ,
                                                    point2      = PhiIndices[1] ,
                                                    point3      = PhiIndices[2] ,
                                                    point4      = PhiIndices[3] )
restraints["Phi"] = restraint

# . Optimization.
ConjugateGradientMinimize_SystemGeometry ( molecule                    ,
                                           maximumIterations    = 2000 ,
                                           logFrequency         =  10 ,
                                           rmsGradientTolerance =  0.1 )

# . Define a random number generator in a given state.
normalDeviateGenerator = NormalDeviateGenerator.WithRandomNumberGenerator ( RandomNumberGenerator.WithSeed ( 175189 ) )

# . Heating.
VelocityVerletDynamics_SystemGeometry ( molecule                             ,
                                        logFrequency              =      10 ,
                                        normalDeviateGenerator    = normalDeviateGenerator ,
                                        steps                     =     1000 ,
                                        timeStep                  =    0.001 ,
                                        temperatureScaleFrequency =      100 ,
                                        temperatureScaleOption    = "linear" ,
                                        temperatureStart          =     10.0 ,
                                        temperatureStop           =    300.0 )

# . Equilibration.
VelocityVerletDynamics_SystemGeometry ( molecule                               ,
                                        logFrequency              =        50 ,
                                        steps                     =       5000 ,
                                        timeStep                  =      0.001 ,
                                        temperatureScaleFrequency =        100 ,
                                        temperatureScaleOption    = "constant" ,
                                        temperatureStart          =      300.0 )

# . Data-collection.
trajectory = ExportTrajectory ( os.path.join ( scratchPath, "bala_c7eq_restrained.ptGeo" ), molecule )
VelocityVerletDynamics_SystemGeometry ( molecule             ,
                                        logFrequency =   50 ,
                                        steps        = 10000 ,
                                        timeStep     = 0.001 ,
                                        trajectories = [ ( trajectory, 100 ) ] )

# . Footer.
logFile.Footer ( )
