"""Example 17 + RMSD."""

# . Note that a simple average will not work if the phi or psi angles cross the +/-180 boundary.

from Definitions import *

# . Header.
logFile.Header ( )

# . Read the molecule definition.
molecule = ImportSystem ( os.path.join ( molPath, "bala_c7eq.mol" ) )
molecule.Summary ( )

# . Save the starting coordinates.
reference3 = Clone ( molecule.coordinates3 )

# . Get the atom masses.
masses  = Array.FromIterable ( [ atom.mass for atom in molecule.atoms ] )

# . Define the trajectory.
#trajectory = ImportTrajectory ( os.path.join ( scratchPath, "bala_c7eq.ptGeo" ), molecule )
trajectory = ImportTrajectory ( os.path.join ( scratchPath, "bala_c7eq_restrained.ptGeo" ), molecule )
trajectory.ReadHeader ( )

# . Loop over the frames in the trajectory.
phi = []
psi = []
rms = []
while trajectory.RestoreOwnerData ( ):
    phi.append ( molecule.coordinates3.Dihedral ( 4, 6,  8, 14 ) )
    psi.append ( molecule.coordinates3.Dihedral ( 6, 8, 14, 16 ) )
    # . Determine the RMS coordinate deviation between the initial structure.
    reference3.Superimpose ( molecule.coordinates3, weights = masses )
    rms.append (reference3.RootMeanSquareDeviation ( molecule.coordinates3, weights = masses ) )

# . Finish up.
trajectory.ReadFooter ( )
trajectory.Close ( )

# . Set up the statistics calculation.
phiStatistics = Statistics ( phi )
psiStatistics = Statistics ( psi )
rmsStatistics = Statistics ( rms )

# . Output the results.
table = logFile.GetTable ( columns = [ 20, 20, 20, 20 ] )
table.Start   ( )
table.Title   ( "Phi/Psi Angles and RMSD" )
table.Heading ( "Frame" )
table.Heading ( "Phi"   )
table.Heading ( "Psi"   )
table.Heading ( "RMSD"  )
for ( i, ( h, s, r ) ) in enumerate ( zip ( phi, psi, rms ) ):
    table.Entry ( "{:d}"  .format ( i ) )
    table.Entry ( "{:.2f}".format ( h ) )
    table.Entry ( "{:.2f}".format ( s ) )
    table.Entry ( "{:.2f}".format ( r ) )
table.Entry ( "Mean:",               align = Align.Left )
table.Entry ( "{:.2f}".format ( phiStatistics.mean ) )
table.Entry ( "{:.2f}".format ( psiStatistics.mean ) )
table.Entry ( "{:.2f}".format ( rmsStatistics.mean ) )
table.Entry ( "Standard Deviation:", align = Align.Left )
table.Entry ( "{:.2f}".format ( phiStatistics.standardDeviation ) )
table.Entry ( "{:.2f}".format ( psiStatistics.standardDeviation ) )
table.Entry ( "{:.2f}".format ( rmsStatistics.standardDeviation ) )
table.Stop ( )

# . Footer.
logFile.Footer ( )
